@@ -78,24 +78,28 @@ class CouplingMode(Enum):
     UNI_DIRECTIONAL_READ_COUPLING = 6
 
 
-def determine_function_type(input_obj):
+def determine_function_type(input_obj, dims):
     """
     Determines if the function is scalar- or vector-valued based on rank evaluation.
 
     Parameters
     ----------
-    input_obj :
-        A FEniCS function.
+    input_obj : FEniCS Function or FEniCS FunctionSpace
+        A FEniCS Function object or a FEniCS FunctionSpace object
+    dims : int
+        Dimension of problem.
 
     Returns
     -------
     tag : bool
         0 if input_function is SCALAR and 1 if input_function is VECTOR.
     """
-    if isinstance(input_obj, FunctionSpace):  # scalar-valued functions have rank 0 is FEniCS
-        if input_obj.num_sub_spaces() == 0:
+    if isinstance(input_obj, FunctionSpace):
+        obj_dim = input_obj.num_sub_spaces()
+        if obj_dim == 0:
             return FunctionType.SCALAR
-        elif input_obj.num_sub_spaces() == 2:
+        elif obj_dim == 2 or obj_dim == 3:
+            assert obj_dim == dims
             return FunctionType.VECTOR
     elif isinstance(input_obj, Function):
         if input_obj.value_rank() == 0:
@@ -120,7 +124,7 @@ def filter_point_sources(point_sources, filter_out, warn_duplicate=True):
     filter_out: FEniCS domain
         Defines the domain where PointSources should be filtered out.
     warn_duplicate: bool
-        Set False to surpress warnings, if double-boundary points are filtered out.
+        Set False to suppress warnings, if double-boundary points are filtered out.
 
     Returns
     -------
@@ -218,10 +222,7 @@ def get_fenics_vertices(function_space, coupling_subdomain, dims):
         if coupling_subdomain.inside(v.point(), True):
             lids.append(v.index())
             gids.append(v.global_index())
-            if dims == 2:
-                coords.append([v.x(0), v.x(1)])
-            if dims == 3:
-                coords.append([v.x(0), v.x(1), v.x(2)])
+            coords.append([v.x(d) for d in range(dims)])
 
     return np.array(lids), np.array(gids), np.array(coords)
 
@@ -280,13 +281,8 @@ def get_owned_vertices(function_space, coupling_subdomain, dims):
     # Get coordinates and global IDs of all vertices of the mesh  which lie on the coupling boundary.
     # These vertices include shared (owned + unowned) and non-shared vertices in a parallel setting
     gids, lids, coords = [], [], []
-    coord = None
     for v in coupling_vertices:
-        if dims == 2:
-            coord = [v.x(0), v.x(1)]
-        elif dims == 3:
-            coord = [v.x(0), v.x(1), v.x(2)]
-
+        coord = [v.x(d) for d in range(dims)]
         for dof in dofs:
             if (dof == coord).all():
                 gids.append(v.global_index())
@@ -344,14 +340,9 @@ def get_unowned_vertices(function_space, coupling_subdomain, dims):
     # Get coordinates and global IDs of all vertices of the mesh  which lie on the coupling boundary.
     # These vertices include shared (owned + unowned) and non-shared vertices in a parallel setting
     gids = []
-    coord = None
     for v in coupling_verts:
         ownership = False
-        if dims == 2:
-            coord = [v.x(0), v.x(1)]
-        elif dims == 3:
-            coord = [v.x(0), v.x(1), v.x(2)]
-
+        coord = [v.x(d) for d in range(dims)]
         for dof in dofs:
             if (dof == coord).all():
                 ownership = True
@@ -386,12 +377,12 @@ def get_coupling_boundary_edges(function_space, coupling_subdomain, global_ids,
         Array of second vertex of each edge.
     """
 
-    def edge_is_on(subdomain, edge):
+    def edge_is_on(subdomain, this_edge):
         """
         Check whether edge lies within subdomain
         """
-        assert(len(list(vertices(edge))) == 2)
-        return all([subdomain.inside(v.point(), True) for v in vertices(edge)])
+        assert(len(list(vertices(this_edge))) == 2)
+        return all([subdomain.inside(v.point(), True) for v in vertices(this_edge)])
 
     vertices1_ids = []
     vertices2_ids = []
@@ -427,34 +418,34 @@ def get_forces_as_point_sources(fixed_boundary, function_space, data):
 
     Returns
     -------
-    x_forces : list
-        Dictionary carrying X component of forces with reference to each point on the coupling interface.
-    y_forces : list
-        Dictionary carrying Y component of forces with reference to each point on the coupling interface.
+    forces : list
+        D number of lists carrying components of forces with reference to points on the coupling interface.
+        D is dimension of the problem.
     """
-    x_forces = dict()  # dict of PointSources for Forces in x direction
-    y_forces = dict()  # dict of PointSources for Forces in y direction
-
-    fenics_vertices = np.array(list(data.keys()))
+    vertices = np.array(list(data.keys()))
     nodal_data = np.array(list(data.values()))
 
-    # Check for shape of coupling_mesh_vertices and raise Assertion for 3D
-    n_vertices, _ = fenics_vertices.shape
+    n_vertices, dims = vertices.shape
+    assert (dims == 2 or dims == 3), "Provided data does not have the correct dimensions"
 
-    vertices_x = fenics_vertices[:, 0]
-    vertices_y = fenics_vertices[:, 1]
+    forces = []
+    for d in range(dims):
+        forces.append(dict())
 
     for i in range(n_vertices):
-        px, py = vertices_x[i], vertices_y[i]
-        key = (px, py)
-        x_forces[key] = PointSource(function_space.sub(0), Point(px, py), nodal_data[i, 0])
-        y_forces[key] = PointSource(function_space.sub(1), Point(px, py), nodal_data[i, 1])
+        key = []
+        for d in range(dims):
+            key.append(vertices[i, d])
+        key = tuple(key)
+
+        for d in range(dims):
+            forces[d][key] = PointSource(function_space.sub(d), Point(key), nodal_data[i, d])
 
     # Avoid application of PointSource and Dirichlet boundary condition at the same point by filtering
-    x_forces = filter_point_sources(x_forces, fixed_boundary, warn_duplicate=False)
-    y_forces = filter_point_sources(y_forces, fixed_boundary, warn_duplicate=False)
+    for d in range(dims):
+        forces[d] = filter_point_sources(forces[d], fixed_boundary, warn_duplicate=False)
 
-    return x_forces.values(), y_forces.values()  # don't return dictionary, but list of PointSources
+    return (forces[d].values() for d in range(dims))
 
 
 def get_communication_map(comm, function_space, owned_vertices, unowned_vertices):
